<?php
defined('IN_WEB') or die('Include Error!');
/**
 * Created 时间操作函数库
 * User: wsc
 * Date: 2018/12/12
 * Time: 9:45
 */
class timeFuns
{
    /**
     * 当前毫秒数
     *
     */
    public static function nowMsec(){
        //$msec当前毫秒 $sec当前秒时间戳
        list($usec, $sec) = explode(" ", microtime());
        $msec=round($usec*1000);
        $time = $sec*1000 + $msec;
        return $time;
    }
    /**
     * 返回今日开始和结束的时间戳
     *
     * @return array
     */
    public static function today()
    {
        return [
            mktime(0, 0, 0, date('m'), date('d'), date('Y')),
            mktime(23, 59, 59, date('m'), date('d'), date('Y'))
        ];
    }

    /**
     * 返回昨日开始和结束的时间戳
     *
     * @return array
     */
    public static function yesterday()
    {
        $yesterday = date('d') - 1;
        return [
            mktime(0, 0, 0, date('m'), $yesterday, date('Y')),
            mktime(23, 59, 59, date('m'), $yesterday, date('Y'))
        ];
    }

    /**
     * 返回本周开始和结束的时间戳
     *
     * @return array
     */
    public static function week()
    {
        $timestamp = time();
        return [
            strtotime(date('Y-m-d', strtotime("this week Monday", $timestamp))),
            strtotime(date('Y-m-d', strtotime("this week Sunday", $timestamp))) + 24 * 3600 - 1
        ];
    }

    /**
     * 返回上周开始和结束的时间戳
     *
     * @return array
     */
    public static function lastWeek()
    {
        $timestamp = time();
        return [
            strtotime(date('Y-m-d', strtotime("last week Monday", $timestamp))),
            strtotime(date('Y-m-d', strtotime("last week Sunday", $timestamp))) + 24 * 3600 - 1
        ];
    }

    /**
     * 返回本月开始和结束的时间戳
     *
     * @return array
     */
    public static function month($everyDay = false)
    {
        return [
            mktime(0, 0, 0, date('m'), 1, date('Y')),
            mktime(23, 59, 59, date('m'), date('t'), date('Y'))
        ];
    }

    /**
     * 返回上个月开始和结束的时间戳
     *
     * @return array
     */
    public static function lastMonth()
    {
        $begin = mktime(0, 0, 0, date('m') - 1, 1, date('Y'));
        $end = mktime(23, 59, 59, date('m') - 1, date('t', $begin), date('Y'));

        return [$begin, $end];
    }

    /**
     * 返回今年开始和结束的时间戳
     *
     * @return array
     */
    public static function year()
    {
        return [
            mktime(0, 0, 0, 1, 1, date('Y')),
            mktime(23, 59, 59, 12, 31, date('Y'))
        ];
    }

    /**
     * 返回去年开始和结束的时间戳
     *
     * @return array
     */
    public static function lastYear()
    {
        $year = date('Y') - 1;
        return [
            mktime(0, 0, 0, 1, 1, $year),
            mktime(23, 59, 59, 12, 31, $year)
        ];
    }

    public static function dayOf()
    {

    }

    /**
     * 获取几天前零点到现在/昨日结束的时间戳
     *
     * @param int $day 天数
     * @param bool $now 返回现在或者昨天结束时间戳
     * @return array
     */
    public static function dayToNow($day = 1, $now = true)
    {
        $end = time();
        if (!$now) {
            list($foo, $end) = self::yesterday();
        }

        return [
            mktime(0, 0, 0, date('m'), date('d') - $day, date('Y')),
            $end
        ];
    }

    /**
     * 返回几天前的时间戳
     *
     * @param int $day
     * @return int
     */
    public static function daysAgo($day = 1)
    {
        $nowTime = time();
        return $nowTime - self::daysToSecond($day);
    }

    /**
     * 返回几天后的时间戳
     *
     * @param int $day
     * @return int
     */
    public static function daysAfter($day = 1)
    {
        $nowTime = time();
        return $nowTime + self::daysToSecond($day);
    }

    /**
     * 天数转换成秒数
     *
     * @param int $day
     * @return int
     */
    public static function daysToSecond($day = 1)
    {
        return $day * 86400;
    }

    /**
     * 周数转换成秒数
     *
     * @param int $week
     * @return int
     */
    public static function weekToSecond($week = 1)
    {
        return self::daysToSecond() * 7 * $week;
    }

    /**
     * 毫秒转分秒毫秒
     * @param $msec
     * @return string
     */
    public static function msecToMinutes($msec){
        $i = floor(floor($msec/1000)/60);
        $s = floor(($msec-$i*60*1000)/1000);
        $ms = round(($msec%1000)/10);
        $arr = ['i'=>$i,'s'=>$s,'ms'=>$ms];
//        $str = $i.'`'.$s.'``'.$ms.'```';
        return $arr;
    }
    private static function startTimeToEndTime()
    {

    }

    /**
     * 判断周末
     * @param $str
     * @return bool
     */
    public static function isWeek($str){
        if((date('w',strtotime($str))==6) || (date('w',strtotime($str)) == 0)){
            return true;
        }else{
            return false;
        }
    }

    /**
     * 时间遍历，返回范围内时间
     * @param $date1
     * @param $date2
     * @param $t
     * @return array
     */
    public static function date_range($date1,$date2,$t='Ymd'){
        $timestamp1=strtotime($date1);
        $timestamp2=strtotime($date2);
        $days=($timestamp2-$timestamp1)/86400+1;
        $date=[];
        for($i=0;$i<$days;$i++){
            $date[]=date($t,$timestamp1+(86400*$i));
        }
        return $date;
    }

    /**
     * 今天过去分钟数
     * @param $time
     * @return false|float|int|string
     * Created by: Owen
     * Created on: 2020/4/24 19:50
     */
    public static function todaySecond($time = null){
        if($time == null){
            $time = time();
        }
        $hour   = date('G', $time) * 60;
        return $hour + date('i', $time);
    }
}